@@ -78,8 +78,18 @@ $elements['quota'] = array(
     'defaultvalue' => get_config_plugin('artefact', 'file', 'defaultquota'),
 );
 
-$authinstances = auth_get_auth_instances();
-if (count($authinstances) > 1) {
+if ($USER->get('admin')) {
+    $authinstances = auth_get_auth_instances();
+} else {
+    $admininstitutions = $USER->get('admininstitutions');
+    $authinstances = auth_get_auth_instances_for_institutions($admininstitutions);
+    if (empty($authinstances)) {
+        $SESSION->add_info_msg(get_string('configureauthplugin', 'admin'));
+        redirect(get_config('wwwroot').'admin/users/institutions.php?i='.key($admininstitutions).'&amp;edit=1');
+    }
+}
+
+if (count($authinstances) > 0) {
     $options = array();
 
     $external = false;
@@ -109,11 +119,6 @@ if (count($authinstances) > 1) {
             'description'  => get_string('remoteusernamedescription', 'admin'),
         );
     }
-} else if (count($authinstances == 1)) {
-    $elements['authinstance'] = array(
-        'type'         => 'hidden',
-        'value'        => $authinstances[0]->id,
-    );
 }
 
 $elements['submit'] = array(
@@ -161,7 +166,7 @@ function adduser_validate(Pieform $form, $values) {
         $form->set_error('username', get_string('addusererrorinvalidusername', 'admin'));
         return;
     }
-    if (record_exists('usr', 'username', $username)) {
+    if (!$form->get_error('username') && record_exists_select('usr', 'LOWER(username) = ?', strtolower($username))) {
         $form->set_error('username', get_string('usernamealreadytaken', 'auth.internal'));
         return;
     }
@@ -197,7 +202,7 @@ function adduser_submit(Pieform $form, $values) {
     $user->quota          = $values['quota'];
     $user->passwordchange = 1;
 
-    global $USER;
+    global $USER, $SESSION;
     if ($USER->get('admin')) {  // Not editable by institutional admins
         $user->staff = (int) ($values['staff'] == 'on');
         $user->admin = (int) ($values['admin'] == 'on');
@@ -243,10 +248,15 @@ function adduser_submit(Pieform $form, $values) {
     handle_event('createuser', $user);
     db_commit();
 
-    email_user($user, $USER, get_string('accountcreated'),
-        get_string('accountcreatedchangepasswordtext', 'mahara', $user->firstname, get_config('sitename'), $user->username, $user->password, get_config('sitename')),
-        get_string('accountcreatedchangepasswordhtml', 'mahara', $user->firstname, get_config('sitename'), $user->username, $user->password, get_config('sitename'))
-    );
+    try {
+        email_user($user, $USER, get_string('accountcreated'),
+            get_string('accountcreatedchangepasswordtext', 'mahara', $user->firstname, get_config('sitename'), $user->username, $user->password, get_config('sitename')),
+            get_string('accountcreatedchangepasswordhtml', 'mahara', $user->firstname, get_config('sitename'), $user->username, $user->password, get_config('sitename'))
+        );
+    }
+    catch (EmailException $e) {
+        $SESSION->add_error_msg(get_string('newuseremailnotsent', 'admin'));
+    }
 
     redirect('/admin/users/edit.php?id='.$id);
 }
