@@ -50,7 +50,7 @@ if (!$USER->get('admin')) {
     }
     if (empty($institution) && count($USER->get('admininstitutions')) == 1) {
         redirect(get_config('wwwroot') . 'admin/users/institutions.php?i='
-                 . key($USER->get('institutions')));
+                 . key($USER->get('admininstitutions')));
     }
 }
 
@@ -145,7 +145,9 @@ if ($institution || $add) {
     else {
         $data = new StdClass;
         $data->displayname = '';
-        $data->registerallowed = 1;
+        if (!get_config('usersuniquebyusername')) {
+            $data->registerallowed = 1;
+        }
         $data->theme = 'sitedefault';
         $data->defaultmembershipperiod = null;
         $lockedprofilefields = array();
@@ -196,7 +198,10 @@ if ($institution || $add) {
             ),
             'help'   => true,
         ),
-        'authplugin' => array(
+    );
+
+    if ($USER->get('admin')) {
+        $elements['authplugin'] = array(
             'type'    => 'authlist',
             'title'   => get_string('authplugin', 'admin'),
             'options' => $authinstances,
@@ -206,15 +211,27 @@ if ($institution || $add) {
             'institution' => $institution,
             'help'   => true,
             'ignore' => count($authtypes) == 0 || $institution == ''
-        ),
-        'registerallowed' => array(
+        );
+    }
+
+    if (!$add && empty($authinstances)) {
+        if ($USER->get('admin')) {
+            $SESSION->add_error_msg(get_string('adminnoauthpluginforinstitution', 'admin'));
+        }
+        else {
+            $SESSION->add_error_msg(get_string('noauthpluginforinstitution', 'admin'));
+        }
+    }
+
+    if (!get_config('usersuniquebyusername')) {
+        $elements['registerallowed'] = array(
             'type'         => 'checkbox',
             'title'        => get_string('registrationallowed', 'admin'),
-            'description'  => get_string('registrationalloweddescription', 'admin'),
+            'description'  => get_string('registrationalloweddescription2', 'admin'),
             'defaultvalue' => $data->registerallowed,
             'help'   => true,
-        ),
-    );
+        );
+    }
 
     if (empty($data->name) || $data->name != 'mahara') {
         $elements['defaultmembershipperiod'] = array(
@@ -288,7 +305,7 @@ else {
     // Get a list of institutions
     if (!$USER->get('admin')) { // Filter the list for institutional admins
         $where = '
-        AND i.name IN (' . join(',', array_map('db_quote', $USER->get('admininstitutions'))) . ')';
+        WHERE ii.name IN (' . join(',', array_map('db_quote', $USER->get('admininstitutions'))) . ')';
     }
     else {
         $where = '';
@@ -300,17 +317,33 @@ else {
     }
     $institutions = get_records_sql_assoc('
         SELECT
-            i.name, i.displayname, i.maxuseraccounts, 
-            COUNT(ui.usr) AS members, SUM(ui.staff) AS staff, SUM(ui.admin) AS admins
-        FROM {institution} i
-        LEFT OUTER JOIN {usr_institution} ui ON (ui.institution = i.name)
-        LEFT OUTER JOIN {usr} u ON (u.id = ui.usr) 
-        WHERE (u.deleted = 0 OR u.id IS NULL) ' . $where . '
-        GROUP BY
-            i.name, i.displayname, i.maxuseraccounts
-        ORDER BY i.name = \'mahara\', i.displayname', array());
+            ii.name,
+            ii.displayname,
+            ii.maxuseraccounts,
+            COALESCE(a.members, 0) AS members,
+            COALESCE(a.staff, 0) AS staff,
+            COALESCE(a.admins, 0) AS admins
+        FROM
+            {institution} ii
+            LEFT JOIN
+                (SELECT
+                    i.name, i.displayname, i.maxuseraccounts,
+                    COUNT(ui.usr) AS members, SUM(ui.staff) AS staff, SUM(ui.admin) AS admins
+                FROM
+                    {institution} i
+                    LEFT OUTER JOIN {usr_institution} ui ON (ui.institution = i.name)
+                    LEFT OUTER JOIN {usr} u ON (u.id = ui.usr)
+                WHERE
+                    (u.deleted = 0 OR u.id IS NULL)
+                GROUP BY
+                    i.name, i.displayname, i.maxuseraccounts
+                ) a ON (a.name = ii.name)' . $where . '
+                ORDER BY
+                    ii.name = \'mahara\', ii.displayname', array());
     if (isset($defaultinstmembers)) {
         $institutions['mahara']->members = $defaultinstmembers;
+        $institutions['mahara']->staff   = '';
+        $institutions['mahara']->admins  = '';
     }
     $smarty->assign('institutions', $institutions);
 }
@@ -333,7 +366,15 @@ function institution_submit(Pieform $form, $values) {
 
     $newinstitution->displayname                  = $values['displayname'];
     $newinstitution->authplugin                   = empty($values['authplugin']) ? null : $values['authplugin'];
-    $newinstitution->registerallowed              = ($values['registerallowed']) ? 1 : 0;
+    if (get_config('usersuniquebyusername')) {
+        // Registering absolutely not allowed when this setting is on, it's a 
+        // security risk. See the documentation for the usersuniquebyusername 
+        // setting for more information
+        $newinstitution->registerallowed = 0;
+    }
+    else {
+        $newinstitution->registerallowed              = ($values['registerallowed']) ? 1 : 0;
+    }
     $newinstitution->theme                        = (empty($values['theme']) || $values['theme'] == 'sitedefault') ? null : $values['theme'];
     if ($institution != 'mahara') {
         $newinstitution->defaultmembershipperiod  = ($values['defaultmembershipperiod']) ? intval($values['defaultmembershipperiod']) : null;
@@ -377,6 +418,17 @@ function institution_submit(Pieform $form, $values) {
 
     if ($add) {
         insert_record('institution', $newinstitution);
+        // If registration has been turned on, then we automatically insert an 
+        // internal authentication authinstance
+        if ($newinstitution->registerallowed) {
+            $authinstance = (object)array(
+                'instancename' => 'internal',
+                'priority'     => 0,
+                'institution'  => $newinstitution->name,
+                'authname'     => 'internal',
+            );
+            insert_record('auth_instance', $authinstance);
+        }
     }
     else {
         $where = new StdClass;
@@ -397,19 +449,24 @@ function institution_submit(Pieform $form, $values) {
     db_commit();
 
     if ($add) {
-        $message = get_string('institutionaddedsuccessfully', 'admin');
+        if ($newinstitution->registerallowed) {
+            // If registration is not allowed, then an authinstance will not 
+            // have been created, and thus cause the institution page to add 
+            // its own error message on the next page load
+            $SESSION->add_ok_msg(get_string('institutionaddedsuccessfully2', 'admin'));
+        }
         $nexturl = '/admin/users/institutions.php?i='.urlencode($institution);
     }
     else {
         $message = get_string('institutionupdatedsuccessfully', 'admin');
-        if ($oldtheme != $values['theme']
+        if (isset($values['theme']) && $oldtheme != $values['theme']
             && (!empty($oldtheme) || $values['theme'] != 'sitedefault')) {
             $message .= '  ' . get_string('usersseenewthemeonlogin', 'admin');
         }
+        $SESSION->add_ok_msg($message);
         $nexturl = '/admin/users/institutions.php';
     }
 
-    $SESSION->add_ok_msg($message);
     redirect($nexturl);
 }
 
