@@ -77,7 +77,7 @@ function start_jump_session($peer, $instanceid, $wantsurl="") {
     if (false == $approved) {
         // This shouldn't happen: the user shouldn't have been presented with 
         // the link
-        throw new SystemException('Host not approved for sso');
+        throw new AccessTotallyDeniedException('Host not approved for sso');
     }
 
     // set up the session
@@ -260,6 +260,80 @@ function user_authorise($token, $useragent) {
     return $userdata;
 }
 
+/**
+ * If we're an IDP, kill_children will kill the session of the given user here, 
+ * as well as at any other children
+ *
+ * NOTE: well, currently it doesn't call kill_child on any other children, but 
+ * it will kill the local sessions for the user
+ *
+ * @param   string  $username       Username for session to kill
+ * @param   string  $useragent      SHA1 hash of user agent to look for
+ * @return  string                  A plaintext report of what has happened
+ */
+function kill_children($username, $useragent) {
+    global $REMOTEWWWROOT; // comes from server.php
+    //require_once(get_config('docroot') .'api/xmlrpc/client.php');
+
+    // We've received a logout request for user X. In Mahara, usernames are unique. So we check that user X 
+    // has an authinstance that would have been able to SSO to the remote site.
+    $userid = get_field('usr', 'id', 'username', $username);
+    $providers = get_service_providers(get_field('usr', 'authinstance', 'username', $username));
+
+    $approved = false;
+    foreach ($providers as $provider) {
+        if ($provider['wwwroot'] == $REMOTEWWWROOT) {
+            $approved = true;
+            break;
+        }
+    }
+
+    if (false == $approved) {
+        return 'This host is not permitted to kill sessions for this username';
+    }
+
+    $mnetsessions = get_records_select_array('sso_session', 'userid = ? AND useragent = ?', array($userid, $useragent));
+
+    // Prepare to destroy local sessions associated with the user
+    $start = ob_start();
+    $uc = ini_get('session.use_cookies');
+    ini_set('session.use_cookies', false);
+    $sesscache = isset($_SESSION) ? clone($_SESSION) : null;
+    $sessidcache = session_id();
+    session_write_close();
+    unset($_SESSION);
+
+    foreach($mnetsessions as $mnetsession) {
+        // Kills all local sessions associated with this user
+        // TODO: We should send kill_child requests to the remote servers too
+        session_id($mnetsession->sessionid);
+        session_start();
+        session_unregister("USER");
+        session_unregister("SESSION");
+        unset($_SESSION);
+        $_SESSION = array();
+        session_destroy();
+        session_write_close();
+    }
+
+    // We're done destroying local sessions
+    ini_set('session.use_cookies', $uc);
+    if ($sessidcache) {
+        session_name(get_config('cookieprefix') . 'mahara');
+        session_id($sessidcache);
+        session_start();
+        $_SESSION = ($sesscache) ? clone($sesscache) : null;
+        session_write_close();
+    }
+    $end = ob_end_clean();
+
+    delete_records('sso_session',
+                   'useragent', $useragent,
+                   'userid',    $userid);
+
+    return true;
+}
+
 function xmlrpc_not_implemented() {
     return true;
 }
@@ -279,7 +353,7 @@ function get_service_providers($instance) {
         return $cache[$instance];
     }
 
-    $query = '
+    $query = "
         SELECT
             h.name,
             a.ssolandurl,
@@ -292,20 +366,20 @@ function get_service_providers($instance) {
             {host} h,
             {application} a
         WHERE
-          ((aic.value = 1 AND
-            aic.field = \'theyautocreateusers\' ) OR
+          ((aic.value = '1' AND
+            aic.field = 'theyautocreateusers' ) OR
            (aic.value = ?  AND
-            aic.field = \'parent\')) AND
+            aic.field = 'parent')) AND
 
             aic.instance = aic2.instance AND
-            aic2.field = \'wwwroot\' AND
+            aic2.field = 'wwwroot' AND
             aic2.value = h.wwwroot AND
 
             aic.instance = aic3.instance AND
-            aic3.field = \'wessoout\' AND
-            aic3.value = \'1\' AND
+            aic3.field = 'wessoout' AND
+            aic3.value = '1' AND
 
-            a.name = h.appname';
+            a.name = h.appname";
     try {
         $results = get_records_sql_assoc($query, array('value' => $instance));
     } catch (SQLException $e) {
@@ -352,7 +426,7 @@ function get_public_key($uri, $application=null) {
     curl_setopt($ch, CURLOPT_POST, true);
     curl_setopt($ch, CURLOPT_USERAGENT, 'Moodle');
     curl_setopt($ch, CURLOPT_POSTFIELDS, $rq);
-    curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: text/xml charset=UTF-8"));
+    curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: text/xml charset=UTF-8", 'Expect: '));
 
     $raw = curl_exec($ch);
     if (empty($raw)) {
@@ -420,13 +494,13 @@ function xmlrpc_error($message, $code) {
 EOF;
 }
 
-function xmlenc_envelope_strip(&$xml) {
+function xmlenc_envelope_strip(&$xml, $oldkeyok=false) {
     $openssl           = OpenSslRepo::singleton();
     $payload_encrypted = true;
     $data              = base64_decode($xml->EncryptedData->CipherData->CipherValue);
     $key               = base64_decode($xml->EncryptedKey->CipherData->CipherValue);
     $payload           = '';    // Initialize payload var
-    $payload           = $openssl->openssl_open($data, $key);
+    $payload           = $openssl->openssl_open($data, $key, $oldkeyok);
     $xml               = parse_payload($payload);
     return $payload;
 }
@@ -482,9 +556,33 @@ function xmldsig_envelope_strip(&$xml) {
         } catch (Exception $e) {
             throw new MaharaException('Signed payload is not a valid XML document', 6007);
         }
-    } else {
-        throw new MaharaException('An error occurred while trying to verify your message signature', 6004);
     }
+    else if ($signature_verified == 0) {
+        // Maybe the remote host is using a new key?
+        //$new_public_key = get_public_key($wwwroot, $peer->application->name);
+        // Make a dummy request so we'll be given a new key
+        log_info("Signature verification for message from $wwwroot failed, checking to see if they have a new signature for us");
+        require_once(get_config('docroot') . 'api/xmlrpc/client.php');
+        $client = new Client();
+        $client->set_method('system/listServices')
+               ->send($wwwroot);
+
+        // Now use the new key and re-try verification
+        $peer = get_peer($wwwroot, false);
+        $signature_verified = openssl_verify($payload, $signature, $peer->certificate);
+        if ($signature_verified == 1) {
+            log_info("Succefully retrieved a new key for $wwwroot");
+            // Parse the XML
+            try {
+                $xml = new SimpleXMLElement($payload);
+                return $payload;
+            } catch (Exception $e) {
+                throw new MaharaException('Signed payload is not a valid XML document', 6007);
+            }
+        }
+    }
+
+    throw new MaharaException('An error occurred while trying to verify your message signature', 6004);
 }
 
 /**
@@ -657,10 +755,13 @@ class OpenSslRepo {
      *
      * @param string   $data
      * @param string   $key
+     * @param bool     $oldkeyok If true, we will simply return the data rather 
+     *                           than complaining about the key being old (if 
+     *                           we could decrypt it with an older key)
      * @return string
      * @access public
      */
-    public function openssl_open($data, $key) {
+    public function openssl_open($data, $key, $oldkeyok=false) {
         $payload = '';
         $isOpen = openssl_open($data, $payload, $key, $this->keypair['privatekey']);
 
@@ -674,12 +775,17 @@ class OpenSslRepo {
                 $isOpen      = openssl_open($data, $payload, $key, $keyresource);
                 if ($isOpen) {
                     // It's an older code, sir, but it checks out
-                    // We notify the remote host that the key has changed
-                    throw new CryptException($this->keypair['certificate'], 7025);
+                    if ($oldkeyok) {
+                        return $payload;
+                    }
+                    else {
+                        // We notify the remote host that the key has changed
+                        throw new CryptException($this->keypair['certificate'], 7025);
+                    }
                 }
             }
         }
-        throw new CryptException('Invalid certificate', 7025);
+        throw new CryptException('We know nothing about the key used to encrypt this message', 7025);
     }
 
     /**
@@ -780,25 +886,23 @@ class OpenSslRepo {
      * @return bool     
      * @access private
      */
-    private function get_keypair($regenerate = null) {
+    public function get_keypair($regenerate = null) {
         $this->keypair = array();
         $records       = null;
         
-        if (empty($regenerate)) {
-            $records = get_records_select_menu('config', "field IN ('openssl_keypair', 'openssl_keypair_expires')", 'field', 'field, value');
-            if (!empty($records)) {
-                list($this->keypair['certificate'], $this->keypair['keypair_PEM']) = explode('@@@@@@@@', $records['openssl_keypair']);
-                $this->keypair['expires'] = $records['openssl_keypair_expires'];
-                if ($this->keypair['expires'] <= time()) {
-                    $openssl_history = $this->get_history();
-                    array_unshift($openssl_history, $this->keypair);
-                    $this->save_history($openssl_history);
-                } else {
-                    return true;
-                }
+        if ($records = get_records_select_menu('config', "field IN ('openssl_keypair', 'openssl_keypair_expires')", 'field', 'field, value')) {
+            list($this->keypair['certificate'], $this->keypair['keypair_PEM']) = explode('@@@@@@@@', $records['openssl_keypair']);
+            $this->keypair['expires'] = $records['openssl_keypair_expires'];
+            if (empty($regenerate) && $this->keypair['expires'] > time()) {
+                return true;
             }
         }
 
+        // Save out the old key
+        $openssl_history = $this->get_history();
+        array_unshift($openssl_history, $this->keypair);
+        $this->save_history($openssl_history);
+
         // Initialize a new set of SSL keys
         $this->keypair = array();
         $this->generate_keypair();
@@ -820,12 +924,19 @@ class OpenSslRepo {
         }
 
         if (empty($records)) {
-                   $result = insert_record('config', $keyrecord);
-            return $result & insert_record('config', $expiresrecord);
-        } else {
-                   $result = update_record('config', $keyrecord,     array('field' => 'openssl_keypair'));
-            return $result & update_record('config', $expiresrecord, array('field' => 'openssl_keypair_expires'));
+            db_begin();
+            insert_record('config', $keyrecord);
+            insert_record('config', $expiresrecord);
+            db_commit();
         }
+        else {
+            db_begin();
+            update_record('config', $keyrecord,     array('field' => 'openssl_keypair'));
+            update_record('config', $expiresrecord, array('field' => 'openssl_keypair_expires'));
+            db_commit();
+        }
+        log_info("New public key has been generated. It expires " . date('Y/m/d h:i:s', $credentials['validTo_time_t']));
+        return true;
     }
 
     /**
@@ -870,7 +981,7 @@ class OpenSslRepo {
 
         $new_key = openssl_pkey_new();
         $csr_rsc = openssl_csr_new($dn, $new_key, array('private_key_bits',2048));
-        $selfSignedCert = openssl_csr_sign($csr_rsc, null, $new_key, 28 /*days*/);
+        $selfSignedCert = openssl_csr_sign($csr_rsc, null, $new_key, 365 /*days*/);
         unset($csr_rsc); // Free up the resource
 
         // We export our self-signed certificate to a string.
