@@ -236,7 +236,7 @@ class PluginArtefactFile extends PluginArtefact {
 
             if ((!isset($currentmimetypes[$description]) && $newmimetypes)
                 || ((join('', $currentmimetypes[$description]) != join('', $newmimetypes)))) {
-                log_debug('Need to update mime types for ' . $description);
+                log_debug('Updating mime types for ' . $description);
                 delete_records('artefact_file_mime_types', 'description', $description);
                 foreach ($newmimetypes as $newmimetype) {
                     $record = new StdClass;
@@ -728,6 +728,10 @@ class ArtefactTypeFile extends ArtefactTypeFileBase {
         }
         return false;
     }
+
+    public static function get_quota_usage($artefact) {
+        return get_field('artefact_file_files', 'size', 'artefact', $artefact);
+    }
 }
 
 class ArtefactTypeFolder extends ArtefactTypeFileBase {
@@ -877,6 +881,15 @@ class ArtefactTypeFolder extends ArtefactTypeFileBase {
         return $record->id;
     }
 
+    // append the view id to to the end of image and anchor urls so they are visible to logged out users also
+    public static function append_view_url($postcontent, $view_id) {
+        $postcontent = preg_replace('#(<a[^>]+href="[^>]+artefact/file/download\.php\?file=\d+)#', '\1&amp;view=' . $view_id , $postcontent);
+        $postcontent = preg_replace('#(<img[^>]+src="[^>]+artefact/file/download\.php\?file=\d+)#', '\1&amp;view=' . $view_id, $postcontent);
+        $postcontent = preg_replace('#(<img[^>]+src="([^>]+artefact/file/download\.php\?file=\d+&amp;view=\d+)"[^>]*>)#', '<a href="\2">\1</a>', $postcontent);
+
+        return $postcontent;
+    }
+
     public static function get_links($id) {
         $wwwroot = get_config('wwwroot');
 
