@@ -511,9 +511,6 @@ class ArtefactTypeProfileIcon extends ArtefactTypeProfileField {
             . ($this->get('id') % 256) . '/' . $this->get('id'));
 
         $downloadpath = get_config('wwwroot') . 'thumb.php?type=profileiconbyid&id=' . $this->id;
-        if (isset($options['viewid'])) {
-            $downloadpath .= '&id=' . $options['viewid'];
-        }
         $smarty = smarty_core();
         $smarty->assign('iconpath', $this->get_icon($options));
         $smarty->assign('downloadpath', $downloadpath);
@@ -552,6 +549,11 @@ class ArtefactTypeProfileIcon extends ArtefactTypeProfileField {
     public function in_view_list() {
         return true;
     }
+
+    public static function get_quota_usage($artefact) {
+        return filesize(get_config('dataroot') . 'artefact/internal/profileicons/originals/'
+            . ($artefact % 256) . '/' . $artefact);
+    }
 }
 
 
