@@ -296,6 +296,29 @@ abstract class Auth {
         return true;
     }
 
+    /**
+     * Called when a user is being logged out, either by clicking a logout 
+     * link, their session timing out or some other method where their session 
+     * is explicitly being ended with no more processing to take place on this 
+     * page load.
+     *
+     * You can use $USER->logout() to log a user out but continue page 
+     * processing if necessary. register.php is an example of such a place 
+     * where this happens.
+     *
+     * If you define this hook, you can call $USER->logout() in it if you need 
+     * to before redirecting. Otherwise, it will be called for you once your 
+     * hook has run.
+     *
+     * If you do not explicitly redirect yourself, once this hook is finished 
+     * the user will be redirected to the homepage with a message saying they 
+     * have been logged out successfully.
+     *
+     * This method has no parameters and needs no return value
+     */
+    public function logout() {
+    }
+
 }
 
 
@@ -336,6 +359,16 @@ function auth_setup () {
     // not have a session, this time will be 0.
     $sessionlogouttime = $USER->get('logout_time');
     if ($sessionlogouttime && isset($_GET['logout'])) {
+        // Call the authinstance' logout hook
+        $authinstance = $SESSION->get('authinstance');
+        if ($authinstance) {
+            $authobj = AuthFactory::create($authinstance);
+            $authobj->logout();
+        }
+        else {
+            log_debug("Strange: user " . $USER->get('username') . " had no authinstance set in their session");
+        }
+
         $USER->logout();
         $SESSION->add_ok_msg(get_string('loggedoutok'));
         redirect();
@@ -377,7 +410,30 @@ function auth_setup () {
     }
     else if ($sessionlogouttime > 0) {
         // The session timed out
-        $USER->logout();
+
+        $authinstance = $SESSION->get('authinstance');
+        if ($authinstance) {
+            $authobj = AuthFactory::create($authinstance);
+
+            $mnetuser = 0;
+            if ($SESSION->get('mnetuser') && $authobj->parent) {
+                // We wish to remember that the user is an MNET user - even though 
+                // they're using the local login form
+                $mnetuser = $USER->get('id');
+            }
+
+            $authobj->logout();
+            $USER->logout();
+
+            if ($mnetuser != 0) {
+                $SESSION->set('mnetuser', $mnetuser);
+                $SESSION->set('authinstance', $authinstance);
+            }
+        }
+        else {
+            log_debug("Strange: user " . $USER->get('username') . " had no authinstance set in their session");
+        }
+
         if (defined('JSON')) {
             json_reply('global', get_string('sessiontimedoutreload'), 1);
         }
@@ -686,10 +742,16 @@ function auth_check_required_fields() {
     $elements = array();
 
     foreach(ArtefactTypeProfile::get_mandatory_fields() as $field => $type) {
-        if ($field == 'email' || get_profile_field($USER->get('id'), $field) != null) {
+        if (get_profile_field($USER->get('id'), $field) != null) {
             continue;
         }
 
+        if ($field == 'email') {
+            // Use a text field for their first e-mail address, not the 
+            // emaillist element
+            $type = 'text';
+        }
+
         $elements[$field] = array(
             'type'  => $type,
             'title' => get_string($field, 'artefact.internal'),
@@ -709,6 +771,10 @@ function auth_check_required_fields() {
             $elements[$field]['options'] = getoptions_country();
             $elements[$field]['defaultvalue'] = 'nz';
         }
+
+        if ($field == 'email') {
+            $elements[$field]['rules']['email'] = true;
+        }
     }
 
     if (empty($elements)) { // No mandatory fields that aren't set
@@ -755,7 +821,17 @@ function requiredfields_submit(Pieform $form, $values) {
  */
 function auth_check_password_change() {
     global $USER;
-    if (!$USER->get('passwordchange')) {
+    if (
+        !$USER->get('passwordchange')                                // User doesn't need to change their password
+        || ($USER->get('parentuser') && $USER->get('loginanyway'))   // User is masquerading and wants to log in anyway
+        || defined('NOCHECKPASSWORDCHANGE')                          // The page wants to skip this hassle
+        ) {
+        return;
+    }
+
+    // Check if the user wants to log in anyway
+    if ($USER->get('passwordchange') && $USER->get('parentuser') && isset($_GET['loginanyway'])) {
+        $USER->loginanyway = true;
         return;
     }
 
@@ -810,6 +886,11 @@ function auth_check_password_change() {
 
         $smarty = smarty();
         $smarty->assign('change_password_form', pieform($form));
+        if ($USER->get('parentuser')) {
+            $smarty->assign('loginasoverridepasswordchange',
+                get_string('loginasoverridepasswordchange', 'admin',
+                    '<a href="' . get_config('wwwroot') . '?loginanyway">', '</a>'));
+        }
         $smarty->display('change_password.tpl');
         exit;
     }
@@ -1132,7 +1213,7 @@ function login_submit(Pieform $form, $values) {
                 $USER->authinstance = $authinstance->id;
                 $userdata = $auth->get_user_info($username);
 
-                if (empty($userdata) || empty($userdata->email)) {
+                if (empty($userdata)) {
                     throw new AuthUnknownUserException("\"$username\" is not known");
                 }
                 // We have the data - create the user
@@ -1143,7 +1224,13 @@ function login_submit(Pieform $form, $values) {
                 if (isset($userdata->lastname)) {
                     $USER->lastname = $userdata->lastname;
                 }
-                $USER->email = $userdata->email;
+                if (isset($userdata->email)) {
+                    $USER->email = $userdata->email;
+                }
+                else {
+                    // The user will be asked to populate this when they log in.
+                    $USER->email = null;
+                }
                 try {
                     db_begin();
                     $USER->commit();
@@ -1153,7 +1240,9 @@ function login_submit(Pieform $form, $values) {
                     if (isset($userdata->lastname)) {
                         set_profile_field($USER->id, 'lastname', $USER->lastname);
                     }
-                    set_profile_field($USER->id, 'email', $USER->email);
+                    if (isset($userdata->email)) {
+                        set_profile_field($USER->id, 'email', $USER->email);
+                    }
                     if ($authinstance->institution !== 'mahara') {
                         $USER->join_institution($authinstance->institution);
                     }
@@ -1340,6 +1429,19 @@ function auth_handle_account_expiries() {
 
 }
 
+/**
+ * Clears out old session files
+ *
+ * This should be run once every now and then (once a day is good), to clean 
+ * out session files of users whose sessions have timed out.
+ */
+function auth_remove_old_session_files() {
+    // Note: read the manpage for find and xargs to understand what is going on 
+    // here. In particular, -mtime +1 means files older than about two days 
+    // will be removed
+    exec('find ' . escapeshellarg(get_config('dataroot') . 'sessions') . ' -type f -mtime +1 | xargs -n 1000 -r rm');
+}
+
 function auth_generate_login_form() {
     if (!get_config('installed')) {
         return;
@@ -1375,7 +1477,7 @@ function auth_generate_login_form() {
             ),
             'register' => array(
                 'value' => '<div><a href="' . get_config('wwwroot') . 'register.php" tabindex="2">' . get_string('register') . '</a>'
-                    . '<br><a href="' . get_config('wwwroot') . 'forgotpass.php" tabindex="2">' . get_string('passwordreminder') . '</a></div>'
+                    . '<br><a href="' . get_config('wwwroot') . 'forgotpass.php" tabindex="2">' . get_string('lostusernamepassword') . '</a></div>'
             )
         )
     )));
