@@ -86,6 +86,7 @@ class User {
             'admininstitutions' => array(),
             'staffinstitutions' => array(),
             'parentuser'       => null,
+            'loginanyway'       => false,
             'sesskey'          => ''
         );
         $this->attributes = array();
@@ -122,6 +123,40 @@ class User {
         return $this;
     }
 
+    /**
+     * Populates this object with the user record identified by the given 
+     * username
+     *
+     * @throws AuthUnknownUserException If the user cannot be found. Note that 
+     *                                  deleted users _can_ be found
+     */
+    public function find_by_username($username) {
+
+        if (!is_string($username)) {
+            throw new InvalidArgumentException('username parameter must be a string to create a User object');
+        }
+
+        $sql = 'SELECT
+                    *,
+                    ' . db_format_tsfield('expiry') . ',
+                    ' . db_format_tsfield('lastlogin') . ',
+                    ' . db_format_tsfield('suspendedctime') . '
+                FROM
+                    {usr}
+                WHERE
+                    username = ?';
+
+        $user = get_record_sql($sql, $username);
+
+        if (false == $user) {
+            throw new AuthUnknownUserException("User with username \"$username\" is not known");
+        }
+
+        $this->populate($user);
+        $this->reset_institutions();
+        return $this;
+    }
+
     /**
      * Finds details for a user given a username and their authentication 
      * instance.
@@ -136,37 +171,53 @@ class User {
         }
 
         if ($remoteuser) {
+            // See if the user has either the child or the parent authinstance. 
+            // Most of the time, it's the parent auth instance that is 
+            // stored with the user, but if they were created by (for 
+            // example) SSO with no parent, then it will be the child that 
+            // is stored. Nevertheless, a parent could be added later, and 
+            // that should not matter in finding the user
+            $parentwhere = '';
             if ($parentid = get_field('auth_instance_config', 'value', 'field', 'parent', 'instance', $instanceid)) {
-                // See if the user has either the child or the parent authinstance. 
-                // Most of the time, it's the parent auth instance that is 
-                // stored with the user, but if they were created by (for 
-                // example) SSO with no parent, then it will be the child that 
-                // is stored. Nevertheless, a parent could be added later, and 
-                // that should not matter in finding the user
-                $where = '(r.authinstance = ' . db_quote($instanceid) . ' OR r.authinstance = ' . db_quote($parentid) . ')';
-            }
-            else {
-                $where = 'r.authinstance = ' . db_quote($instanceid);
+                $parentwhere = '
+                            OR
+                            (
+                                LOWER(username) = (
+                                    SELECT
+                                        username
+                                    FROM
+                                        {usr} us
+                                    JOIN
+                                        {auth_remote_user} aru ON (us.id = aru.localusr)
+                                    WHERE
+                                        aru.remoteusername = ' . db_quote($username) . '
+                                        AND us.authinstance = ' . db_quote($parentid) . '
+                                )
+                                AND
+                                u.authinstance = ' . db_quote($parentid) . '
+                            )
+                    ';
             }
+
             $sql = 'SELECT
                         u.*, 
                         ' . db_format_tsfield('u.expiry', 'expiry') . ', 
                         ' . db_format_tsfield('u.lastlogin', 'lastlogin') . ', 
                         ' . db_format_tsfield('u.suspendedctime', 'suspendedctime') . '
                     FROM {usr} u
-                    INNER JOIN {auth_remote_user} r ON u.id = r.localusr
+                    LEFT JOIN {auth_remote_user} r ON u.id = r.localusr
                     WHERE
-                        LOWER(r.remoteusername) = ?
-                        AND u.deleted = 0
-                        AND ' . $where;
-        } else {
-            if ($parentid = get_field('auth_instance_config', 'value', 'field', 'parent', 'instance', $instanceid)) {
-                // See the comment above about checking for the child or parent
-                $where = '(authinstance = ' . db_quote($instanceid) . ' OR authinstance = ' . db_quote($parentid) . ')';
-            }
-            else {
-                $where = 'authinstance = ' . db_quote($instanceid);
-            }
+                        (
+                            (
+                                LOWER(r.remoteusername) = ?
+                                AND r.authinstance = ?
+                            )'
+                            . $parentwhere
+                            . '
+                        )';
+            $user = get_record_sql($sql, array($username, $instanceid));
+        }
+        else {
             $sql = 'SELECT
                         *, 
                         ' . db_format_tsfield('expiry') . ', 
@@ -176,13 +227,10 @@ class User {
                         {usr}
                     WHERE
                         LOWER(username) = ? AND
-                        u.deleted = 0 AND
                         authinstance = ' . db_quote($instanceid);
+            $user = get_record_sql($sql, array($username));
         }
 
-
-        $user = get_record_sql($sql, array($username));
-
         if (false == $user) {
             throw new AuthUnknownUserException("User with username \"$username\" is not known at auth instance \"$instanceid\"");
         }
@@ -516,6 +564,10 @@ class User {
         $this->staffinstitutions  = $staffinstitutions;
     }
 
+    public function can_view_artefact($a) {
+        return $this->get('admin') || $this->get('id') == $a->get('owner');
+    }
+
 }
 
 
@@ -578,7 +630,7 @@ class LiveUser extends User {
         $auth = AuthFactory::create($instanceid);
         if ($auth->authenticate_user_account($user, $password)) {
             $user->lastauthinstance = $auth->instanceid;
-            $this->authenticate($user);
+            $this->authenticate($user, $auth->instanceid);
             return true;
         }
 
@@ -596,6 +648,10 @@ class LiveUser extends User {
         if ($this->authenticated === true) {
             $this->SESSION->set('messages', array());
         }
+
+        // Unset session variables related to authentication
+        $this->SESSION->set('authinstance', null);
+
         reset($this->defaults);
         foreach (array_keys($this->defaults) as $key) {
             $this->set($key, $this->defaults[$key]);
@@ -619,11 +675,23 @@ class LiveUser extends User {
      * When a user creates a security context by whatever method, we do some 
      * standard stuff
      *
-     * @param  object $user     Record from the usr table
+     * @param  object $user          Record from the usr table
+     * @param  integer $authinstance The ID of the authinstance that the user 
+     *                               signed in with
      * @return void
      */
-    protected function authenticate($user) {
+    protected function authenticate($user, $authinstance) {
         $this->authenticated  = true;
+
+        // If the user has reauthenticated and they were an MNET user, we 
+        // don't set these variables, because we wish to remember that they 
+        // originally SSO-ed in from their other authinstance. See the 
+        // session timeout code in auth_setup() for more info.
+        if ($this->SESSION->get('mnetuser') != $user->id) {
+            $this->SESSION->set('mnetuser', null);
+            $this->SESSION->set('authinstance', $authinstance);
+        }
+
         $this->populate($user);
         session_regenerate_id(true);
         $this->lastlogin          = time();
@@ -650,7 +718,7 @@ class LiveUser extends User {
     public function reanimate($id, $instanceid) {
         if ($user = get_record('usr','id',$id)) {
             $user->lastauthinstance = $instanceid;
-            $this->authenticate($user);
+            $this->authenticate($user, $instanceid);
             return true;
         }
         return false;
@@ -665,7 +733,7 @@ class LiveUser extends User {
      */
     public function get($key) {
         if (!array_key_exists($key, $this->defaults)) {
-            throw new InvalidArgumentException($key);
+            throw new InvalidArgumentException("Invalid key: $key");
         }
         if (null !== ($value = $this->SESSION->get("user/$key"))) {
             return $value;
@@ -728,6 +796,7 @@ class LiveUser extends User {
 
         $this->reloadLiveUser($olduser->id);
         $this->set('parentuser', null);
+        $this->set('loginanyway', false);
 
         return $id;
     }
