@@ -57,10 +57,10 @@ class AuthXmlrpc extends Auth {
         $this->config['portno']                = 80;
         $this->config['xmlrpcserverurl']       = '';
         $this->config['changepasswordurl']     = '';
-        $this->config['updateuserinfoonlogin'] = 1;
+        $this->config['updateuserinfoonlogin'] = 0;
         $this->config['weautocreateusers']     = 0;
         $this->config['theyautocreateusers']   = 0;
-        $this->config['wessoout']              = 1;
+        $this->config['wessoout']              = 0;
         $this->config['theyssoin']             = 0;
         $this->config['parent']                = null;
         $this->file = fopen('/tmp/out.txt', 'w');
@@ -100,7 +100,7 @@ class AuthXmlrpc extends Auth {
      * Grab a delegate object for auth stuff
      */
     public function request_user_authorise($token, $remotewwwroot) {
-        global $USER;
+        global $USER, $SESSION;
         $this->must_be_ready();
         $peer = get_peer($remotewwwroot);
 
@@ -130,10 +130,46 @@ class AuthXmlrpc extends Auth {
         // Retrieve a $user object. If that fails, create a blank one.
         try {
             $user = new User;
-            $user->find_by_instanceid_username($this->instanceid, $remoteuser->username, true);
+            if (get_config('usersuniquebyusername')) {
+                // When turned on, this setting means that it doesn't matter 
+                // which other application the user SSOs from, they will be 
+                // given the same account in Mahara.
+                //
+                // This setting is one that has security implications unless 
+                // only turned on by people who know what they're doing. In 
+                // particular, every system linked to Mahara should be making 
+                // sure that same username == same person.  This happens for 
+                // example if two Moodles are using the same LDAP server for 
+                // authentication.
+                //
+                // If this setting is on, it must NOT be possible to self 
+                // register on the site for ANY institution - otherwise users 
+                // could simply pick usernames of people's accounts they wished 
+                // to steal.
+                if ($institutions = get_column('institution', 'name', 'registerallowed', '1')) {
+                    log_warn("usersuniquebyusername is turned on but registration is allowed for an institution. "
+                        . "No institution can have registration allowed for it, for security reasons.\n"
+                        . "The following institutions have registration enabled:\n  " . join("\n  ", $institutions));
+                    throw new AccessDeniedException();
+                }
+
+                if (!get_config('usersallowedmultipleinstitutions')) {
+                    log_warn("usersuniquebyusername is turned on but usersallowedmultipleinstitutions is off. "
+                        . "This makes no sense, as users will then change institution every time they log in from "
+                        . "somewhere else. Please turn this setting on in Site Options");
+                    throw new AccessDeniedException();
+                }
+
+                $user->find_by_username($remoteuser->username);
+            }
+            else {
+                $user->find_by_instanceid_username($this->instanceid, $remoteuser->username, true);
+            }
+
             if ($user->get('suspendedcusr')) {
                 die_info(get_string('accountsuspended', 'mahara', strftime(get_string('strftimedaydate'), $user->get('suspendedctime')), $user->get('suspendedreason')));
             }
+
             if ('1' == $this->config['updateuserinfoonlogin']) {
                 $update = true;
             }
@@ -145,7 +181,10 @@ class AuthXmlrpc extends Auth {
                 }
                 $user = new User;
                 $create = true;
-            } else {
+            }
+            else {
+                log_debug("User authorisation request from $remotewwwroot failed - "
+                    . "remote user '$remoteuser->username' is unknown to us and auto creation of users is turned off");
                 return false;
             }
         }
@@ -231,6 +270,11 @@ class AuthXmlrpc extends Auth {
             $user->commit();
         }
 
+        if (get_config('usersuniquebyusername')) {
+            // Add them to the institution they have SSOed in by
+            $user->join_institution($peer->institution);
+        }
+
 
         // See if we need to create/update a profile Icon image
         if ($create || $update) {
@@ -321,6 +365,14 @@ class AuthXmlrpc extends Auth {
 
         // We know who our user is now. Bring her back to life.
         $USER->reanimate($user->id, $this->instanceid);
+
+        // Set session variables to let the application know this session was 
+        // initiated by MNET. Don't forget that users could initiate their 
+        // sessions without MNET sometimes, which is why this data is stored in 
+        // the session object.
+        $SESSION->set('mnetuser', $user->id);
+        $SESSION->set('authinstance', $this->instanceid);
+
         return true;
     }
 
@@ -424,6 +476,76 @@ class AuthXmlrpc extends Auth {
         }
     }
 
+    public function kill_parent($username) {
+        require_once(get_config('docroot') . 'api/xmlrpc/client.php');
+
+        // For some people, the call to kill_children fails (when the remote 
+        // site is a Moodle). We still haven't worked out why that is, but it's 
+        // not a problem on the Mahara site
+        try {
+            $client = new Client();
+            $client->set_method('auth/mnet/auth.php/kill_children')
+                   ->add_param($username)
+                   ->add_param(sha1($_SERVER['HTTP_USER_AGENT']))
+                   ->send($this->wwwroot);
+        }
+        catch (XmlrpcClientException $e) {
+            log_debug("XMLRPC error occured while calling MNET method kill_children on $this->wwwroot");
+            log_debug("This means that single-signout probably didn't work properly, but the problem "
+                . "is at the remote application");
+            log_debug("If the remote application is Moodle, and you're happy with a Mahara developer "
+                . "getting access to your system so they can try and debug the problem, get in touch with dev@mahara.org");
+            log_debug("Exception message follows:");
+            log_debug($e->getMessage());
+        }
+    }
+
+    /**
+     * Overrides the default logout mechanism to do proper single singout
+     */
+    public function logout() {
+        global $USER, $SESSION;
+
+        if (get_config('usersuniquebyusername')) {
+            // The auth_remote_user will have a row for the institution in 
+            // which the user SSOed into first. However, they could have 
+            // been coming from somewhere else this time, which is why we 
+            // can't use auth_remote_user for the lookup. Their username 
+            // won't change for their Mahara account anyway, so just grab 
+            // it out of the usr table.
+            $remoteusername = get_field('usr', 'username', 'id', $USER->get('id'));
+        }
+        else {
+            // Check the auth_remote_user table for what the remote 
+            // application thinks the username is
+            $remoteusername = get_field('auth_remote_user', 'remoteusername', 'localusr', $USER->get('id'), 'authinstance', $this->instanceid);
+            if (!$remoteusername && $this->parent) {
+                $remoteusername = get_field('auth_remote_user', 'remoteusername', 'localusr', $USER->get('id'), 'authinstance', $this->parent);
+            }
+        }
+
+        $USER->logout();
+
+        if (isset($_GET['logout'])) {
+            // Explicit logout request
+            $this->kill_parent($remoteusername);
+            redirect($this->wwwroot);
+        }
+        elseif (!$this->parent) {
+            $this->kill_parent($remoteusername);
+            // Redirect back to their IDP if they don't have a parent auth method set 
+            // (aka: they can't log in at Mahara's log in form)
+            $peer = get_peer($this->wwwroot);
+            // TODO: This should be stored in the application config table
+            $jumpurl = str_replace('land', 'jump', $peer->application->ssolandurl);
+            redirect($this->wwwroot . $jumpurl . '?hostwwwroot=' . dropslash(get_config('wwwroot')) . '&wantsurl=' . urlencode($_SERVER['REQUEST_URI']));
+        }
+
+        // Anything else is a session timeout
+
+        $SESSION->set('mnetuser', null);
+    }
+
 }
 
 /**
@@ -641,7 +763,7 @@ class PluginAuthXmlrpc extends PluginAuth {
             'collapseifoneoption' => true,
             'multiple'            => false,
             'options'             => $apparray,
-            'defaultvalue'        => empty($tmpappname)? key($apparray) : $tmpappname,
+            'defaultvalue'        => empty($tmpappname)? 'moodle' : $tmpappname,
             'help'                => true
         );
 
@@ -653,6 +775,7 @@ class PluginAuthXmlrpc extends PluginAuth {
                 'integer'  => true
             ),
             'defaultvalue' => $peer->portno,
+            'size'   => 4,
             'help'   => true
         );
 
@@ -698,6 +821,9 @@ class PluginAuthXmlrpc extends PluginAuth {
     }
 
     public static function validate_config_options($values, $form) {
+        if (false === strpos($values['wwwroot'], '://')) {
+            $values['wwwroot'] = 'http://' . $values['wwwroot'];
+        }
 
         $authinstance = new stdClass();
         $peer = new Peer();
@@ -723,6 +849,9 @@ class PluginAuthXmlrpc extends PluginAuth {
     }
 
     public static function save_config_options($values, $form) {
+        if (false === strpos($values['wwwroot'], '://')) {
+            $values['wwwroot'] = 'http://' . $values['wwwroot'];
+        }
 
         db_begin();
         $authinstance = new stdClass();
