@@ -180,7 +180,7 @@ $string['adminfilespagedescription'] = 'Here you can upload files that can be in
 // Networking options
 $string['networkingextensionsmissing'] = 'Sorry, you cannot configure Mahara networking because your PHP installation is missing one or more required extensions:';
 $string['publickey'] = 'Public key';
-$string['publickeydescription'] = 'This public key is automatically generated, and rotated every 28 days';
+$string['publickeydescription2'] = 'This public key is automatically generated, and rotated every %s days';
 $string['publickeyexpires'] = 'Public key expires';
 $string['enablenetworkingdescription'] = 'Allow your Mahara server to communicate with servers running Moodle and other applications';
 $string['enablenetworking'] = 'Enable networking';
@@ -215,13 +215,24 @@ $string['uploadcsverrorinvalidusername'] = 'Error on line %s of your file: The u
 $string['uploadcsverrormandatoryfieldnotspecified'] = 'Line %s of the file does not have the required "%s" field';
 $string['uploadcsverroruseralreadyexists'] = 'Line %s of the file specifies the username "%s" that already exists';
 $string['uploadcsverroremailaddresstaken'] = 'Line %s of the file specifies the e-mail address "%s" that is already taken by another user';
-$string['uploadcsvpagedescription'] = '<p>You may use this facility to upload new users via a <acronym title="Comma Separated Values">CSV</acronym> file.</p>
+$string['uploadcsvpagedescription2'] = '<p>You may use this facility to upload new users via a <acronym title="Comma Separated Values">CSV</acronym> file.</p>
    
 <p>The first row of your CSV file should specify the format of your CSV data. For example, it should look like this:</p>
 
 <pre>username,password,email,firstname,lastname,studentid</pre>
 
-<p>This row must include the <tt>username</tt>, <tt>password</tt>, <tt>email</tt>, <tt>firstname</tt> and <tt>lastname</tt> fields always, and then any fields that are both mandatory and locked for the institution you are uploading the users for. You can <a href="%s">configure the mandatory fields</a> for all institutions, or <a href="%s">configure the locked fields for each institution</a>.</p>
+<p>This row must include the <tt>username</tt>, <tt>password</tt>, <tt>email</tt>, <tt>firstname</tt> and <tt>lastname</tt> fields. It must also includes fields that you have made mandatory for all users to fill out, and any fields locked for the institution you are uploading the users for. You can <a href="%s">configure the mandatory fields</a> for all institutions, or <a href="%s">configure the locked fields for each institution</a>.</p>
+
+<p>Your CSV file may include any other profile fields as you require. The full list of fields is:</p>
+
+%s';
+$string['uploadcsvpagedescription2institutionaladmin'] = '<p>You may use this facility to upload new users via a <acronym title="Comma Separated Values">CSV</acronym> file.</p>
+
+<p>The first row of your CSV file should specify the format of your CSV data. For example, it should look like this:</p>
+
+<pre>username,password,email,firstname,lastname,studentid</pre>
+
+<p>This row must include the <tt>username</tt>, <tt>password</tt>, <tt>email</tt>, <tt>firstname</tt> and <tt>lastname</tt> fields. It must also include any fields that the site administrator has made mandatory, and any fields that are locked for your institution. You can <a href="%s">configure the locked fields</a> for the institution(s) you manage.</p>
 
 <p>Your CSV file may include any other profile fields as you require. The full list of fields is:</p>
 
@@ -293,6 +304,7 @@ $string['adduser'] = 'Add User';
 $string['adduserdescription'] = 'Create a new user';
 $string['adduserpagedescription'] = '<p>Here you may add a new user to the system. Once added, they will be sent an e-mail informing them of their new account, including their username and password. They will be asked to change their password upon first log in.</p>';
 $string['createuser'] = 'Create User';
+$string['newuseremailnotsent'] = 'Failed to send welcome email to new user.';
 
 // Login as
 $string['loginasuser'] = 'Login as %s';
@@ -301,22 +313,26 @@ $string['becomeadminagain'] = 'Become %s again';
 $string['loginasdenied'] = 'Attempt to login as another user without permission';
 $string['loginastwice'] = 'Attempt to login as another user when already logged in as another user';
 $string['loginasrestorenodata'] = 'No user data to restore';
+$string['loginasoverridepasswordchange'] = 'As you are masquerading as another user, you may choose to %slog in anyway%s, ignoring the password change screen.';
 
 // Institutions
-$string['admininstitutions'] = 'Admininster Institutions';
-$string['adminauthorities'] = 'Admininster Authorities';
+$string['Add'] = 'Add';
+$string['admininstitutions'] = 'Administer Institutions';
+$string['adminauthorities'] = 'Administer Authorities';
 $string['addinstitution'] = 'Add Institution';
 $string['authplugin'] = 'Authentication plugin';
 $string['deleteinstitution'] = 'Delete Institution';
 $string['deleteinstitutionconfirm'] = 'Are you really sure you wish to delete this institution?';
-$string['institutionaddedsuccessfully'] = 'Institution added successfully. Please configure an authentication plugin for this institution.';
+$string['institutionaddedsuccessfully2'] = 'Institution added successfully';
 $string['institutiondeletedsuccessfully'] = 'Institution deleted successfully';
+$string['noauthpluginforinstitution'] = 'Your site administrator has not configured an authentication plugin for this institution.';
+$string['adminnoauthpluginforinstitution'] = 'Please configure an authentication plugin for this institution.';
 $string['institutionname'] = 'Institution name';
 $string['institutionnamealreadytaken'] = 'This institution name is already taken';
 $string['institutiondisplayname'] = 'Institution display name';
 $string['institutionupdatedsuccessfully'] = 'Institution updated successfully.';
 $string['registrationallowed'] = 'Registration allowed?';
-$string['registrationalloweddescription'] = 'Whether users can register for the system with this institution';
+$string['registrationalloweddescription2'] = 'Whether users can register for your site for this institution using the registration form';
 $string['defaultmembershipperiod'] = 'Default membership period';
 $string['defaultmembershipperioddescription'] = 'How long new members remain associated with the institution';
 $string['authenticatedby'] = 'Authentication Method';
