@@ -190,7 +190,14 @@ function activity_process_queue() {
                 }
                 $viewsnotified[$data->view] = true;
             }
-            handle_activity($activity->type, $data, true);
+            try {
+                handle_activity($activity->type, $data, true);
+            }
+            catch (MaharaException $e) {
+                // Exceptions can happen while processing the queue, we just 
+                // log them and continue
+                log_debug($e->getMessage());
+            }
         }
         delete_records('activity_queue');
     }
@@ -199,7 +206,7 @@ function activity_process_queue() {
 
 function activity_get_viewaccess_users($view, $owner, $type) {
     $type = activity_locate_typerecord($type);
-    $sql = 'SELECT userid, u.*, p.method, ap.value AS lang
+    $sql = "SELECT userid, u.*, p.method, ap.value AS lang
                 FROM (
                 SELECT (CASE WHEN usr1 = ? THEN usr2 ELSE usr1 END) AS userid 
                     FROM {usr_friend} f
@@ -220,9 +227,8 @@ function activity_get_viewaccess_users($view, $owner, $type) {
                         WHERE ag.view = ? AND g.deleted = 0
                 ) AS userlist
                 JOIN {usr} u ON u.id = userlist.userid
-                LEFT JOIN {usr_activity_preference} p ON p.usr = u.id
-                LEFT OUTER JOIN {usr_account_preference} ap ON (ap.usr = u.id AND ap.field = \'lang\')
-            WHERE p.activity = ?';
+                LEFT JOIN {usr_activity_preference} p ON p.usr = u.id AND p.activity = ?
+                LEFT JOIN {usr_account_preference} ap ON ap.usr = u.id AND ap.field = 'lang'";
     $values = array($owner, $owner, $owner, 'friends', $view, $view, $view, 0, 1, $view, $type->id);
     if (!$u = get_records_sql_assoc($sql, $values)) {
         $u = array();
