@@ -28,7 +28,7 @@ defined('INTERNAL') || die();
 
 function xmldb_core_upgrade($oldversion=0) {
     ini_set('max_execution_time', 120); // Let's be safe
-    ini_set('memory_limit', '64M');
+    raise_memory_limit('64M');
 
     $status = true;
 
@@ -875,6 +875,171 @@ function xmldb_core_upgrade($oldversion=0) {
         }
     }
 
+    // We discovered that username case insensitivity was not being enforced at 
+    // most of the entry points to the system at which users can be created. 
+    // This problem manifested itself as users who had the same LOWER(username) 
+    // as another not being able to log in. The fix is to implement the checks, 
+    // rename the "duplicate" users and add a constraint on the database so it 
+    // can't happen again
+    if ($oldversion < 2008040202) {
+        $renamed = $newusernames = $oldusernames = array();
+        $allusers = get_records_array('usr', '', '', 'id', 'id, username');
+
+        $usernamemapping = array();
+        foreach ($allusers as $user) {
+            $oldusernames[] = $user->username;
+            $usernamemapping[strtolower($user->username)][] = array('id' => $user->id, 'username' => $user->username);
+        }
+
+        foreach ($usernamemapping as $lcname => $users) {
+            if (count($users) == 1) {
+                continue;
+            }
+
+            // Uhohes. Rename the user(s) who were created last
+            $skippedfirst = false;
+            foreach ($users as $user) {
+                if (!$skippedfirst) {
+                    $skippedfirst = true;
+                    continue;
+                }
+
+                $userobj = new User();
+                $userobj->find_by_id($user['id']);
+
+                // Append digits keeping total length <= 30
+                $i = 1;
+                $newname = substr($user['username'], 0, 29) . $i;
+                while (isset($newusernames[$newname]) || isset($oldusernames[$newname])) {
+                    $i++;
+                    $newname = substr($user['username'], 0, 30 - floor(log10($i)+1)) . $i;
+                }
+                set_field('usr', 'username', $newname, 'id', $user['id']);
+                $newusernames[$newname] = true;
+
+                $renamed[$newname] = $userobj;
+                log_debug(" * Renamed {$user['username']} to $newname");
+            }
+        }
+
+        if (!empty($renamed)) {
+            // Notify changed usernames to administrator
+            $report = '# Each line in this file is in the form "old_username new_username"'."\n";
+            $message = "Mahara now requires usernames to be unique, case insensitively.\n";
+            $message .= "Some usernames on your site were changed during the upgrade:\n\n";
+            foreach ($renamed as $newname => $olduser) {
+                $report .= "$olduser->username $newname\n";
+                $message .= "Old username: $olduser->username\n"
+                    . "New username: $newname\n\n";
+            }
+            $sitename = get_config('sitename');
+            $file = get_config('dataroot') . 'user_migration_report_2.txt';
+            if (file_put_contents($file, $report)) {
+                $message .= "\n" . 'A copy of this list has been saved to the file ' . $file;
+            }
+            global $USER;
+            email_user($USER, null, $sitename . ': User migration', $message);
+            // Notify changed usernames to users
+            $usermessagestart = "Your username at $sitename has been changed:\n\n";
+            $usermessageend = "\n\nNext time you visit the site, please login using your new username.";
+            foreach ($renamed as $newname => $olduser) {
+                if ($olduser->email == '') {
+                    continue;
+                }
+                log_debug("Attempting to notify $newname ($olduser->email) of their new username...");
+                email_user($olduser, null, $sitename . ': User name changed', $usermessagestart
+                           . "Old username: $olduser->username\nNew username: $newname"
+                           . $usermessageend);
+            }
+        }
+
+        // Now we know all usernames are unique over their lowercase values, we 
+        // can put an index in so data doesn't get all inconsistent next time
+        if (is_postgres()) {
+            execute_sql('DROP INDEX {usr_use_uix}');
+            execute_sql('CREATE UNIQUE INDEX {usr_use_uix} ON {usr}(LOWER(username))');
+        }
+        else {
+            // MySQL cannot create indexes over functions of columns. Too bad 
+            // for it. We won't drop the existing index because that offers a 
+            // large degree of protection, but when MySQL finally supports this 
+            // we will be able to add it
+        }
+
+
+        // Install a cron job to delete old session files
+        $cron = new StdClass;
+        $cron->callfunction = 'auth_remove_old_session_files';
+        $cron->minute       = '30';
+        $cron->hour         = '20';
+        $cron->day          = '*';
+        $cron->month        = '*';
+        $cron->dayofweek    = '*';
+        insert_record('cron', $cron);
+    }
+
+    if ($oldversion < 2008040203) {
+        // Install a cron job to recalculate user quotas
+        $cron = new StdClass;
+        $cron->callfunction = 'recalculate_quota';
+        $cron->minute       = '15';
+        $cron->hour         = '2';
+        $cron->day          = '*';
+        $cron->month        = '*';
+        $cron->dayofweek    = '*';
+        insert_record('cron', $cron);
+    }
+
+    if ($oldversion < 2008040204) {
+        if (is_postgres()) {
+            execute_sql('ALTER TABLE {usr_friend_request} RENAME COLUMN reason TO message');
+        }
+        else if (is_mysql()) {
+              execute_sql('ALTER TABLE {usr_friend_request} CHANGE reason message TEXT');
+        }
+    }
+
+    if ($oldversion < 2008040206) {
+        // NOTE: Yes, this number is bigger than the number for the next upgrade
+        // The next upgrade got committed first. It deletes all users properly, 
+        // but the usr table has a 30 character limit on username, which can be 
+        // violated when people with long usernames are deleted
+        $table = new XMLDBTable('usr');
+        $field = new XMLDBField('username');
+        $field->setAttributes(XMLDB_TYPE_CHAR, 100, null, XMLDB_NOTNULL);
+        change_field_precision($table, $field);
+    }
+
+    if ($oldversion < 2008040205) {
+        // Some cleanups for deleted users, based on the new model of handling them
+        if ($userids = get_column('usr', 'id', 'deleted', 1)) {
+            require_once('user.php');
+            foreach ($userids as $userid) {
+                delete_user($userid);
+            }
+        }
+    }
+
+    if ($oldversion < 2008040211) {
+        // Set 'sendemail' flag to true. Mahara 1.1 will set this in 
+        // config-defaults.php
+        set_config('sendemail', '1');
+    }
+
+    if ($oldversion < 2008040212) {
+        // Delete all activity_queue entries older than 2 weeks. Designed to 
+        // prevent total spammage caused by the activity queue processing bug
+        delete_records_select('activity_queue', 'ctime < ?', array(db_format_timestamp(time() - (86400 * 14))));
+    }
+
+    if ($oldversion < 2008040217) {
+        // Make the "port" column larger so it can handle any port number
+        $table = new XMLDBTable('host');
+        $field = new XMLDBField('portno');
+        $field->setAttributes(XMLDB_TYPE_INTEGER, 10, null, XMLDB_NOTNULL, null, null, null, 80);
+        change_field_precision($table, $field);
+    }
+
     return $status;
 
 }
