@@ -59,8 +59,11 @@ function ensure_sanity() {
     default:
         throw new ConfigSanityException(get_string('unknowndbtype', 'error'));
     }
+    if (!extension_loaded('xml')) {
+        throw new ConfigSanityException(get_string('xmlextensionnotloaded', 'error', 'xml'));
+    }
     if (!extension_loaded('libxml')) {
-        throw new ConfigSanityException(get_string('libxmlextensionnotloaded', 'error'));
+        throw new ConfigSanityException(get_string('xmlextensionnotloaded', 'error', 'libxml'));
     }
     if (!extension_loaded('gd')) {
         throw new ConfigSanityException(get_string('gdextensionnotloaded', 'error'));
@@ -138,9 +141,12 @@ function ensure_sanity() {
         !check_dir_exists(get_config('dataroot') . 'smarty/compile') ||
         !check_dir_exists(get_config('dataroot') . 'smarty/cache') ||
         !check_dir_exists(get_config('dataroot') . 'sessions') ||
+        !check_dir_exists(get_config('dataroot') . 'langpacks') ||
         !check_dir_exists(get_config('dataroot') . 'htmlpurifier')) {
         throw new ConfigSanityException(get_string('couldnotmakedatadirectories', 'error'));
     }
+
+    raise_memory_limit('32M');
 }
 
 /**
@@ -302,11 +308,11 @@ function get_string_location($identifier, $section, $variables, $replacefunc='fo
     }
 
     // Define the locations of language strings for this section
-    $docroot = get_config('docroot');
-    $locations = array();
+    $langstringroot = get_language_root();
+    $langdirectory  = ''; // The directory in which the language file for this string should ideally reside, if the language has implemented it
     
     if (false === strpos($section, '.')) {
-        $locations[] = $docroot . 'lang/';
+        $langdirectory = 'lang/';
     }
     else {
         $extras = plugin_types(); // more later..
@@ -316,24 +322,21 @@ function get_string_location($identifier, $section, $variables, $replacefunc='fo
                 if ($tocheck == 'blocktype' && 
                     strpos($pluginname, '/') !== false) { // it belongs to an artefact plugin
                     $bits = explode('/', $pluginname);
-                    $locations[] = $docroot . 'artefact/' . $bits[0] . '/blocktype/' . $bits[1] . '/lang/';
+                    $langdirectory = 'artefact/' . $bits[0] . '/blocktype/' . $bits[1] . '/lang/';
                     $section = 'blocktype.' . $bits[1];
                 }
                 else {
-                    $locations[] = $docroot . $tocheck . '/' . $pluginname . '/lang/';
+                    $langdirectory = $tocheck . '/' . $pluginname . '/lang/';
                 }
             }
         }
     }
 
     // First check all the normal locations for the string in the current language
-    foreach ($locations as $location) {
-        //if local directory not found, or particular string does not exist in local direcotry
-        $langfile = $location . $lang . '/' . $section . '.php';
-        if (is_readable($langfile)) {
-            if ($result = get_string_from_file($identifier, $langfile)) {
-                return $replacefunc($result, $variables);
-            }
+    $langfile = $langstringroot . $langdirectory . $lang . '/' . $section . '.php';
+    if (is_readable($langfile)) {
+        if ($result = get_string_from_file($identifier, $langfile)) {
+            return $replacefunc($result, $variables);
         }
     }
 
@@ -344,28 +347,23 @@ function get_string_location($identifier, $section, $variables, $replacefunc='fo
     }
 
     // Is a parent language defined?  If so, try to find this string in a parent language file
-    foreach ($locations as $location) {
-        $langfile = $location . $lang . '/langconfig.php';
-        if (is_readable($langfile)) {
-            if ($parentlang = get_string_from_file('parentlanguage', $langfile)) {
-                $langfile = $location . $parentlang . '/' . $section . '.php';
-                if (is_readable($langfile)) {
-                    if ($result = get_string_from_file($identifier, $langfile)) {
-                        return $replacefunc($result, $variables);
-                    }
+    $langfile = $langstringroot . 'lang/' . $lang . '/langconfig.php';
+    if (is_readable($langfile)) {
+        if ($parentlang = get_string_from_file('parentlanguage', $langfile)) {
+            $langfile = get_language_root($parentlang) . 'lang/' . $parentlang . '/' . $section . '.php';
+            if (is_readable($langfile)) {
+                if ($result = get_string_from_file($identifier, $langfile)) {
+                    return $replacefunc($result, $variables);
                 }
             }
         }
     }
 
     /// Our only remaining option is to try English
-    foreach ($locations as $location) {
-        //if local_en not found, or string not found in local_en
-        $langfile = $location . 'en.utf8/' . $section . '.php';
-        if (is_readable($langfile)) {
-            if ($result = get_string_from_file($identifier, $langfile)) {
-                return $replacefunc($result, $variables);
-            }
+    $langfile = get_config('docroot') . $langdirectory . 'en.utf8/' . $section . '.php';
+    if (is_readable($langfile)) {
+        if ($result = get_string_from_file($identifier, $langfile)) {
+            return $replacefunc($result, $variables);
         }
     }
 
@@ -378,31 +376,114 @@ function get_string_location($identifier, $section, $variables, $replacefunc='fo
  *
  */
 function get_languages() {
-    $langs = array();
-    $langbase = get_config('docroot') . 'lang/';
-    if (!$langdir = opendir($langbase)) {
-        throw new SystemException('Unable to read language directory '.$langbase);
-    }
-    while (false !== ($subdir = readdir($langdir))) {
-        $langfile = $langbase . $subdir . '/langconfig.php';
-        if ($subdir != "." && $subdir != ".." && is_readable($langfile)) {
-            if ($langname = get_string_from_file('thislanguage',$langfile)) {
-                $langs[$subdir] = $langname;
+    static $langs = array();
+
+    if (!$langs) {
+        foreach (language_get_searchpaths() as $searchpath) {
+            $langbase = $searchpath . 'lang/';
+            if ($langdir = @opendir($langbase)) {
+                while (false !== ($subdir = readdir($langdir))) {
+                    if (preg_match('/\.utf8$/', $subdir) && is_dir($langbase . $subdir)) {
+                        $langfile = $langbase . $subdir . '/langconfig.php';
+                        if (is_readable($langfile)) {
+                            if ($langname = get_string_from_file('thislanguage', $langfile)) {
+                                $langs[$subdir] = $langname;
+                            }
+                        }
+                    }
+                }
+                closedir($langdir);
+            }
+            else {
+                log_warn('Unable to read language directory ' . $langbase);
             }
         }
     }
-    closedir($langdir);
+
     return $langs;
 }
 
+/**
+ * Returns whether the given language is installed/available for use
+ */
 function language_installed($lang) {
-    return is_readable(get_config('docroot') . 'lang/' . $lang . '/langconfig.php');
+    foreach (language_get_searchpaths() as $searchpath) {
+        if (is_readable($searchpath . 'lang/' . $lang . '/langconfig.php')) {
+            return true;
+        }
+    }
+    return false;
 }
 
 /**
- * Return a list of available themes
- * Need to add the theme names sometime; for now use get_string().
+ * Returns a list of directories in which to search for language packs.
  *
+ * This is influenced by the configuration variable 'langpacksearchpaths'
+ */
+function language_get_searchpaths() {
+    static $searchpaths = array();
+
+    if (!$searchpaths) {
+        // Construct the search path
+        $docrootpath = array(get_config('docroot'));
+
+        // Paths to language files in dataroot
+        $datarootpaths = (array)glob(get_config('dataroot') . 'langpacks/*.utf8', GLOB_MARK | GLOB_ONLYDIR);
+
+        // langpacksearchpaths configuration variable - for experts :)
+        $lpsearchpaths = (array)get_config('langpacksearchpaths');
+        $langpacksearchpaths = array();
+        foreach ($lpsearchpaths as $path) {
+            if (is_dir($path)) {
+                $langpacksearchpaths[] = (substr($path, -1) == '/') ? $path : "$path/";
+            }
+            else {
+                log_warn("Path in langpacksearchpaths is not a directory: $path");
+            }
+        }
+
+        $searchpaths = array_merge($docrootpath, $datarootpaths, $langpacksearchpaths);
+    }
+
+    return $searchpaths;
+}
+
+/**
+ * Get the directory in which the specified language pack resides.
+ *
+ * Defaults to getting the directory for the current_language() - i.e. the 
+ * language the user is using
+ *
+ * Returns null if the language can't be found
+ *
+ * @param string $language The language to look for
+ */
+function get_language_root($language=null) {
+    static $language_root_cache = array();
+
+    if (!isset($language_root_cache[$language])) {
+        if ($language == null) {
+            $language = current_language();
+        }
+
+        foreach (language_get_searchpaths() as $path) {
+            if (is_dir("$path/lang/$language")) {
+                return $language_root_cache[$language] = $path;
+            }
+        }
+
+        // Oh noes, can't be found
+        $language_root_cache[$language] = null;
+
+    }
+
+    return $language_root_cache[$language];
+}
+
+/**
+ * Return a list of available themes.
+ *
+ * Themes _must_ have a config.php, and it should specify a theme name.
  */
 function get_themes() {
     $themes = array();
@@ -412,8 +493,6 @@ function get_themes() {
     }
     while (false !== ($subdir = readdir($themedir))) {
         if ($subdir != "." && $subdir != ".." && is_dir($themebase . $subdir)) {
-            $themes[$subdir] = $subdir;
-
             $config_path = $themebase . $subdir . '/config.php';
             if (is_readable($config_path)) {
                 require($config_path);
@@ -1697,6 +1776,76 @@ function microtime_diff($a, $b) {
     return $b_sec - $a_sec + $b_dec - $a_dec;
 }
 
+/**
+ * Function to raise the memory limit to a new value.
+ * Will respect the memory limit if it is higher, thus allowing
+ * settings in php.ini, apache conf or command line switches
+ * to override it
+ *
+ * The memory limit should be expressed with a string (eg:'64M')
+ *
+ * @param string $newlimit the new memory limit
+ * @return bool Whether we were able to raise the limit or not
+ */
+function raise_memory_limit ($newlimit) {
+    if (empty($newlimit)) {
+        return false;
+    }
+
+    $cur = @ini_get('memory_limit');
+    if (empty($cur)) {
+        // If php is compiled without --enable-memory-limits
+        // apparently memory_limit is set to ''
+        $cur=0;
+    }
+    else {
+        if ($cur == -1){
+            return true; // unlimited mem!
+        }
+        $cur = get_real_size($cur);
+    }
+
+    $new = get_real_size($newlimit);
+    if ($new > $cur) {
+        ini_set('memory_limit', $newlimit);
+        return true;
+    }
+    return false;
+}
+
+/**
+ * Converts numbers like 10M into bytes.
+ *
+ * @param string $size The size to be converted
+ * @return integer
+ * @throws SystemException if the string does not have a valid suffix.
+ *                         See the function definition for allowed suffixes.
+ */
+function get_real_size($size=0) {
+    if (!$size) {
+        return 0;
+    }
+    $scan = array(
+        'MB' => 1048576,
+        'Mb' => 1048576,
+        'M'  => 1048576,
+        'm'  => 1048576,
+        'KB' => 1024,
+        'Kb' => 1024,
+        'K'  => 1024,
+        'k'  => 1024,
+    );
+
+    while (list($key) = each($scan)) {
+        if (strlen($size) > strlen($key) && substr($size, -strlen($key)) == $key) {
+            $size = substr($size, 0, -strlen($key)) * $scan[$key];
+            return $size;
+        }
+    }
+
+    throw new SystemException('get_real_size called without valid size suffix');
+}
+
 /**
  * Converts bytes into display form
  *
@@ -1735,15 +1884,22 @@ function display_size($size) {
  * creates the profile sideblock
  */
 function profile_sideblock() {
-    global $USER;
+    global $USER, $SESSION;
     safe_require('notification', 'internal');
     require_once('group.php');
     $data = array(
-        'id' => $USER->get('id'),
+        'id'          => $USER->get('id'),
+        'profileicon' => $USER->get('profileicon') ? $USER->get('profileicon') : 0,
     );
-    $saveduser = $USER->get('parentuser');
-    if (!empty($saveduser) && $saveduser->name) {
-        $data['realuser'] = $saveduser->name;
+
+    if ($SESSION->get('mnetuser')) {
+        $authinstance = $SESSION->get('authinstance');
+        if ($authinstance) {
+            $authobj = AuthFactory::create($authinstance);
+            $peer = get_peer($authobj->wwwroot);
+            $data['mnetloggedinfrom'] = get_string('youhaveloggedinfrom', 'auth.xmlrpc',
+                $authobj->wwwroot, $peer->name);
+        }
     }
     $data['unreadnotifications'] = call_static_method(generate_class_name('notification', 'internal'), 'unread_count', $USER->get('id'));
     $data['unreadnotificationsmessage'] = $data['unreadnotifications'] == 1 ? get_string('unreadmessage') : get_string('unreadmessages');
@@ -1772,4 +1928,38 @@ function profile_sideblock() {
     return $data;
 }
 
+/**
+ * Cronjob to recalculate how much quota each user is using and update it as 
+ * appropriate.
+ *
+ * This gives a backstop for the possibility that there is a bug elsewhere that 
+ * has caused the quota count to get out of sync
+ */
+function recalculate_quota() {
+    if (!$artefacts = get_records_array('artefact', '', '', '', 'id, artefacttype, owner')) {
+        // Nothing to do
+        return;
+    }
+
+    $userquotas = array();
+
+    foreach ($artefacts as $artefact) {
+        safe_require('artefact', get_field('artefact_installed_type', 'plugin', 'name', $artefact->artefacttype));
+        if (!isset($userquotas[$artefact->owner])) {
+            $userquotas[$artefact->owner] = 0;
+        }
+        $userquotas[$artefact->owner] += call_static_method(generate_artefact_class_name($artefact->artefacttype), 'get_quota_usage', $artefact->id);
+    }
+
+    foreach ($userquotas as $user => $quota) {
+        $data = (object) array(
+            'quotaused' => $quota
+        );
+        $where = (object) array(
+            'id' => $user
+        );
+        update_record('usr', $data, $where);
+    }
+}
+
 ?>
