@@ -536,6 +536,15 @@ function core_postinst() {
     set_config('searchplugin', 'internal');
 
     set_config('lang', 'en.utf8');
+
+    // PostgreSQL supports indexes over functions of columns, MySQL does not. 
+    // So we can improve the index on the username field of the usr table for 
+    // Postgres
+    if (is_postgres()) {
+        execute_sql('DROP INDEX {usr_use_uix}');
+        execute_sql('CREATE UNIQUE INDEX {usr_use_uix} ON {usr}(LOWER(username))');
+    }
+
     return $status;
 }
 
@@ -679,6 +688,8 @@ function core_install_firstcoredata_defaults() {
         'auth_clean_partial_registrations'       => array('5', '0', '*', '*', '*'),
         'auth_handle_account_expiries'           => array('5', '10', '*', '*', '*'),
         'activity_process_queue'                 => array('*/5', '*', '*', '*', '*'),
+        'auth_remove_old_session_files'          => array('30', '20', '*', '*', '*'),
+        'recalculate_quota'                      => array('15', '2', '*', '*', '*'),
     );
     foreach ($cronjobs as $callfunction => $times) {
         $cron = new StdClass;
