@@ -241,7 +241,16 @@ function set_profile_field($userid, $field, $value) {
  * @todo, this needs to be better (fix email behaviour)
  */
 function get_profile_field($userid, $field) {
-    $value = get_field('artefact', 'title', 'owner', $userid, 'artefacttype', $field);
+    if ($field == 'email') {
+        $value = get_field_sql("
+            SELECT a.title
+            FROM {usr} u
+            JOIN {artefact} a ON (a.title = u.email AND a.owner = u.id)
+            WHERE a.artefacttype = 'email' AND u.id = ?", array($userid));
+    }
+    else {
+        $value = get_field('artefact', 'title', 'owner', $userid, 'artefacttype', $field);
+    }
 
     if ($value) {
         return $value;
@@ -261,6 +270,11 @@ function get_profile_field($userid, $field) {
  * @throws EmailException
  */ 
 function email_user($userto, $userfrom, $subject, $messagetext, $messagehtml='') {
+    if (!get_config('sendemail')) {
+        // You can entirely disable Mahara from sending any e-mail via the 
+        // 'sendemail' configuration variable
+        return true;
+    }
 
     if (empty($userto)) {
         throw new InvalidArgumentException("empty user given to email_user");
@@ -270,7 +284,8 @@ function email_user($userto, $userfrom, $subject, $messagetext, $messagehtml='')
 
     $mail = new phpmailer();
 
-    $mail->Version = 'Mahara ' . get_config('release'); 
+    // Leaving this commented out - there's no reason for people to know this
+    //$mail->Version = 'Mahara ' . get_config('release');
     $mail->PluginDir = get_config('libroot')  . 'phpmailer/';
     
     $mail->CharSet = 'UTF-8';
@@ -311,8 +326,20 @@ function email_user($userto, $userfrom, $subject, $messagetext, $messagehtml='')
 
     $mail->Subject = substr(stripslashes($subject), 0, 78);
 
-    $usertoname = display_name($userto, $userto);
-    $mail->AddAddress($userto->email, $usertoname );
+    if ($to = get_config('sendallemailto')) {
+        // Admins can configure the system to send all email to a given address 
+        // instead of whoever would receive it, useful for debugging.
+        $mail->addAddress($to);
+        $notice = get_string('debugemail', 'mahara', display_name($userto, $userto), $userto->email);
+        $messagetext =  $notice . "\n\n" . $messagetext;
+        if ($messagehtml) {
+            $messagehtml = '<p>' . hsc($notice) . '</p>' . $messagehtml;
+        }
+    }
+    else {
+        $usertoname = display_name($userto, $userto);
+        $mail->AddAddress($userto->email, $usertoname );
+    }
 
     $mail->WordWrap = 79;   
 
@@ -630,17 +657,30 @@ function unsuspend_user($userid) {
 /**
  * Deletes a user
  *
+ * This function ensures that a user is deleted according to how Mahara wants a 
+ * deleted user to be. You can call it multiple times on the same user without 
+ * harm.
+ *
  * @param int $userid The ID of the user to delete
  */
 function delete_user($userid) {
     db_begin();
 
-    $emailsuffix = '.deleted.' . time();
+    // We want to append 'deleted.timestamp' to some unique fields in the usr 
+    // table, so they can be reused by new accounts
+    $fieldstomunge = array('username', 'email');
+    $datasuffix = '.deleted.' . time();
+
+    $user = get_record('usr', 'id', $userid, null, null, null, null, implode(', ', $fieldstomunge));
 
     $deleterec = new StdClass;
     $deleterec->id = $userid;
     $deleterec->deleted = 1;
-    $deleterec->email = get_field('usr', 'email', 'id', $userid) . $emailsuffix;
+    foreach ($fieldstomunge as $field) {
+        if (!preg_match('/\.deleted\.\d+$/', $user->$field)) {
+            $deleterec->$field = $user->$field . $datasuffix;
+        }
+    }
 
     // Set authinstance to default internal, otherwise the old authinstance can be blocked from deletion
     // by deleted users.
@@ -656,10 +696,31 @@ function delete_user($userid) {
     // 'deleted.[timestamp]'
     execute_sql('UPDATE {artefact_internal_profile_email}
     SET email = email || ?
-    WHERE owner = ?', array($emailsuffix, $userid));
+    WHERE owner = ?', array($datasuffix, $userid));
 
     // mark all groups the user owns as deleted
-    set_field('group', 'deleted', '1', 'owner', $userid);
+    if ($groupids = get_column('group', 'id', 'owner', $userid)) {
+        require_once('group.php');
+        foreach ($groupids as $groupid) {
+            delete_group($groupid);
+        }
+    }
+
+    // Remove user from any groups they're in, invited to or want to be in
+    delete_records('group_member', 'member', $userid);
+    delete_records('group_member_request', 'member', $userid);
+    delete_records('group_member_invite', 'member', $userid);
+
+    // Remove any friend relationships the user is in
+    execute_sql('DELETE FROM {usr_friend}
+        WHERE usr1 = ?
+        OR usr2 = ?', array($userid, $userid));
+    execute_sql('DELETE FROM {usr_friend_request}
+        WHERE owner = ?
+        OR requester = ?', array($userid, $userid));
+
+    // Remove remote user records
+    delete_records('auth_remote_user', 'localusr', $userid);
 
     db_commit();
 
@@ -847,7 +908,7 @@ function get_users_data($userlist) {
                 NULL AS friend,
                 NULL AS requestedfriendship,
                 (SELECT title FROM {artefact} WHERE artefacttype = \'introduction\' AND owner = u.id) AS introduction,
-                reason
+                message
                 FROM {usr} u
                 JOIN {usr_friend_request} fr ON fr.requester = u.id
                 WHERE fr.owner = ?
