@@ -434,8 +434,18 @@ EOF;
 
         usort($SIDEBLOCKS, create_function('$a,$b', 'if ($a["weight"] == $b["weight"]) return 0; return ($a["weight"] < $b["weight"]) ? -1 : 1;'));
 
-        $smarty->assign('userauthinstance', $USER->lastauthinstance);
+        $smarty->assign('userauthinstance', $SESSION->get('authinstance'));
+        $smarty->assign('MNETUSER', $SESSION->get('mnetuser'));
         $smarty->assign('SIDEBLOCKS', $SIDEBLOCKS);
+
+        if ($USER->get('parentuser')) {
+            $smarty->assign('USERMASQUERADING', true);
+            $smarty->assign('masqueradedetails', get_string('youaremasqueradingas', 'mahara', display_name($USER)));
+            $smarty->assign('becomeyouagain',
+                ' <a href="' . hsc($wwwroot) . 'admin/users/changeuser.php?restore=1">'
+                . get_string('becomeadminagain', 'admin', $USER->get('parentuser')->name)
+                . '</a>');
+        }
     }
 
     return $smarty;
@@ -1818,7 +1828,7 @@ function searchform() {
         'elements'            => array(
             'query' => array(
                 'type'           => 'text',
-                'defaultvalue'   => get_string('search'),
+                'defaultvalue'   => get_string('searchusers'),
                 'class'          => 'emptyonfocus'
             ),
             'submit' => array(
@@ -1981,17 +1991,17 @@ function clean_text($text) {
     $config = HTMLPurifier_Config::createDefault();
     $config->set('Cache', 'SerializerPath', get_config('dataroot') . 'htmlpurifier');
 
-    $config->set('HTML', 'DefinitionID', 'Mahara customisations to default config');
-    // NOTE: this MUST be incremented if you change the configuration 
-    // definition - Talk to Nigel about it
-    $config->set('HTML', 'DefinitionRev', 1);
-
-    // This disables caching of HTMLPurifier objects. Worth having off for 
-    // development, but see note above once you're done messing with things
-    //$config->set('Core', 'DefinitionCache', null);
-
     $config->set('Core', 'Encoding', 'UTF-8');
     $config->set('HTML', 'Doctype', 'XHTML 1.0 Transitional');
+    $config->set('AutoFormat', 'Linkify', true);
+
+    // These settings help identify the configuration definition. If the 
+    // definition (the $def object below) is changed (e.g. new method calls 
+    // made on it), the DefinitionRev needs to be increased. See
+    // http://htmlpurifier.org/live/configdoc/plain.html#HTML.DefinitionID
+    $config->set('HTML', 'DefinitionID', 'Mahara customisations to default config');
+    $config->set('HTML', 'DefinitionRev', 1);
+
     $def =& $config->getHTMLDefinition(true);
     $def->addAttribute('a', 'target', 'Enum#_blank,_self,_target,_top');
     $purifier = new HTMLPurifier($config);
